@@ -1,20 +1,23 @@
 """Platform config client."""
 from pkg_resources import get_distribution
 
-from .client import ConfigClient
+from .client import ConfigClient, ConfigClientBase
 from .entities import (
     ACMEEnvironment,
     ARecord,
     AWSCloudProvider,
     AWSCredentials,
     AWSStorage,
+    AWSStorageOptions,
     AzureCloudProvider,
     AzureCredentials,
     AzureReplicationType,
     AzureStorage,
+    AzureStorageOptions,
     AzureStorageTier,
     BucketsConfig,
     CloudProvider,
+    CloudProviderOptions,
     CloudProviderType,
     Cluster,
     ClusterLocationType,
@@ -29,6 +32,7 @@ from .entities import (
     GoogleCloudProvider,
     GoogleFilestoreTier,
     GoogleStorage,
+    GoogleStorageOptions,
     GrafanaCredentials,
     HelmRegistryConfig,
     IdleJobConfig,
@@ -38,7 +42,7 @@ from .entities import (
     MonitoringConfig,
     NeuroAuthConfig,
     NodePool,
-    NodePoolTemplate,
+    NodePoolOptions,
     NodeRole,
     NotificationType,
     OnPremCloudProvider,
@@ -50,11 +54,14 @@ from .entities import (
     Resources,
     SecretsConfig,
     SentryCredentials,
+    Storage,
     StorageConfig,
     StorageInstance,
+    StorageOptions,
     TPUPreset,
     TPUResource,
     VCDCloudProvider,
+    VCDCloudProviderOptions,
     VCDCredentials,
     VCDStorage,
     VolumeConfig,
@@ -62,18 +69,22 @@ from .entities import (
 
 __all__ = [
     "ConfigClient",
+    "ConfigClientBase",
     "ACMEEnvironment",
     "ARecord",
     "AWSCloudProvider",
     "AWSCredentials",
     "AWSStorage",
+    "AWSStorageOptions",
     "AzureCloudProvider",
     "AzureCredentials",
     "AzureReplicationType",
     "AzureStorage",
+    "AzureStorageOptions",
     "AzureStorageTier",
     "BucketsConfig",
     "CloudProvider",
+    "CloudProviderOptions",
     "CloudProviderType",
     "Cluster",
     "ClusterLocationType",
@@ -88,6 +99,7 @@ __all__ = [
     "GoogleCloudProvider",
     "GoogleFilestoreTier",
     "GoogleStorage",
+    "GoogleStorageOptions",
     "GrafanaCredentials",
     "HelmRegistryConfig",
     "IdleJobConfig",
@@ -97,7 +109,7 @@ __all__ = [
     "MonitoringConfig",
     "NeuroAuthConfig",
     "NodePool",
-    "NodePoolTemplate",
+    "NodePoolOptions",
     "NodeRole",
     "NotificationType",
     "OnPremCloudProvider",
@@ -109,11 +121,14 @@ __all__ = [
     "Resources",
     "SecretsConfig",
     "SentryCredentials",
+    "Storage",
     "StorageConfig",
     "StorageInstance",
+    "StorageOptions",
     "TPUPreset",
     "TPUResource",
     "VCDCloudProvider",
+    "VCDCloudProviderOptions",
     "VCDCredentials",
     "VCDStorage",
     "VolumeConfig",
