@@ -26,6 +26,68 @@ class CloudProviderType(str, enum.Enum):
     VCD_MTS = "vcd_mts"
     VCD_SELECTEL = "vcd_selectel"
 
+    @property
+    def is_vcd(self) -> bool:
+        return self.startswith("vcd_")
+
+
+@dataclass(frozen=True)
+class CloudProviderOptions:
+    type: CloudProviderType
+    node_pools: list[NodePoolOptions]
+    storages: list[StorageOptions]
+
+
+@dataclass(frozen=True)
+class VCDCloudProviderOptions(CloudProviderOptions):
+    kubernetes_node_pool_id: str
+    platform_node_pool_id: str
+    url: URL | None = None
+    organization: str | None = None
+    edge_name_template: str | None = None
+    edge_external_network_name: str | None = None
+    catalog_name: str | None = None
+    storage_profile_names: list[str] | None = None
+
+
+@dataclass(frozen=True)
+class NodePoolOptions:
+    id: str
+    machine_type: str
+    cpu: float
+    available_cpu: float
+    memory: int
+    available_memory: int
+    gpu: int | None = None
+    gpu_model: str | None = None
+
+
+@dataclass(frozen=True)
+class StorageOptions:
+    id: str
+
+
+@dataclass(frozen=True)
+class GoogleStorageOptions(StorageOptions):
+    tier: GoogleFilestoreTier
+    min_capacity: int
+    max_capacity: int
+
+
+@dataclass(frozen=True)
+class AWSStorageOptions(StorageOptions):
+    performance_mode: EFSPerformanceMode
+    throughput_mode: EFSThroughputMode
+    provisioned_throughput_mibps: int | None = None
+
+
+@dataclass(frozen=True)
+class AzureStorageOptions(StorageOptions):
+    tier: AzureStorageTier
+    replication_type: AzureReplicationType
+    min_file_share_size: int
+    max_file_share_size: int
+
 
 class NodeRole(str, enum.Enum):
     KUBERNETES = "kubernetes"
@@ -36,35 +98,36 @@ class NodeRole(str, enum.Enum):
 @dataclass(frozen=True)
 class NodePool:
     name: str
+    id: str | None = None
     role: NodeRole = NodeRole.PLATFORM_JOB
 
     min_size: int = 0
     max_size: int = 1
-    idle_size: int = 0
+    idle_size: int | None = None
 
     machine_type: str | None = None
-    cpu: float = 1
-    available_cpu: float = 1
-    memory_mb: int = 1024
-    available_memory_mb: int = 1024
+    cpu: float | None = None
+    available_cpu: float | None = None
+    memory: int | None = None
+    available_memory: int | None = None
 
-    disk_size_gb: int = 20
+    disk_size: int | None = None
     disk_type: str | None = None
 
     gpu: int | None = None
     gpu_model: str | None = None
 
-    price: Decimal = Decimal()
+    price: Decimal | None = None
     currency: str | None = None
 
-    is_preemptible: bool = False
+    is_preemptible: bool | None = None
 
-    zones: Sequence[str] = field(default_factory=tuple)
+    zones: tuple[str, ...] | None = None
 
 
 @dataclass(frozen=True)
 class StorageInstance:
-    size_mb: int | None = None
+    size: int | None = None
     name: str | None = None
     ready: bool = False
 
@@ -222,7 +285,7 @@ class VCDCredentials:
 class VCDStorage(Storage):
     description: str
     profile_name: str
-    size_gib: int
+    size: int
 
 
 @dataclass(frozen=True)
@@ -319,7 +382,7 @@ class CredentialsConfig:
 
 @dataclass(frozen=True)
 class VolumeConfig:
-    size_mb: int | None = None
+    size: int | None = None
     path: str | None = None
 
 
@@ -352,7 +415,7 @@ class SecretsConfig:
 @dataclass(frozen=True)
 class DisksConfig:
     url: URL
-    storage_limit_per_user_gb: int
+    storage_limit_per_user: int
 
 
 @dataclass(frozen=True)
@@ -390,7 +453,7 @@ class ResourcePreset:
     name: str
     credits_per_hour: Decimal
     cpu: float
-    memory_mb: int
+    memory: int
     gpu: int | None = None
     gpu_model: str | None = None
     tpu: TPUPreset | None = None
@@ -407,9 +470,9 @@ class ResourcePoolType:
     idle_size: int = 0
     cpu: float = 1.0
     available_cpu: float = 1.0
-    memory_mb: int = 1024
-    available_memory_mb: int = 1024
-    disk_size_gb: int = 150
+    memory: int = 2**30  # 1gb
+    available_memory: int = 2**30  # 1gb
+    disk_size: int = 150 * 2**30  # 150gb
     gpu: int | None = None
     gpu_model: str | None = None
     price: Decimal = Decimal()
@@ -421,7 +484,7 @@ class ResourcePoolType:
 @dataclass(frozen=True)
 class Resources:
     cpu_m: int
-    memory_mb: int
+    memory: int
     gpu: int = 0
 
 
