@@ -12,13 +12,16 @@ from .entities import (
     AWSCloudProvider,
     AWSCredentials,
     AWSStorage,
+    AWSStorageOptions,
     AzureCloudProvider,
     AzureCredentials,
     AzureReplicationType,
     AzureStorage,
+    AzureStorageOptions,
     AzureStorageTier,
     BucketsConfig,
     CloudProvider,
+    CloudProviderOptions,
     CloudProviderType,
     Cluster,
     ClusterLocationType,
@@ -33,6 +36,7 @@ from .entities import (
     GoogleCloudProvider,
     GoogleFilestoreTier,
     GoogleStorage,
+    GoogleStorageOptions,
     GrafanaCredentials,
     HelmRegistryConfig,
     IdleJobConfig,
@@ -43,7 +47,7 @@ from .entities import (
     MonitoringConfig,
     NeuroAuthConfig,
     NodePool,
-    NodePoolTemplate,
+    NodePoolOptions,
     NodeRole,
     OnPremCloudProvider,
     OpenStackCredentials,
@@ -56,9 +60,11 @@ from .entities import (
     SentryCredentials,
     StorageConfig,
     StorageInstance,
+    StorageOptions,
     TPUPreset,
     TPUResource,
     VCDCloudProvider,
+    VCDCloudProviderOptions,
     VCDCredentials,
     VCDStorage,
     VolumeConfig,
@@ -66,6 +72,101 @@ from .entities import (
 
 
 class EntityFactory:
+    @classmethod
+    def create_cloud_provider_options(
+        cls, type: CloudProviderType, payload: dict[str, Any]
+    ) -> CloudProviderOptions:
+        if type.is_vcd:
+            return cls._create_vcd_cloud_provider_options(type, payload)
+        return CloudProviderOptions(
+            type=type,
+            node_pools=[
+                cls.create_node_pool_options(p) for p in payload.get("node_pools", ())
+            ],
+            storages=[
+                cls.create_storage_options(type, p) for p in payload.get("storages", ())
+            ],
+        )
+
+    @classmethod
+    def _create_vcd_cloud_provider_options(
+        cls, type: CloudProviderType, payload: dict[str, Any]
+    ) -> VCDCloudProviderOptions:
+        url = payload.get("url")
+        return VCDCloudProviderOptions(
+            type=type,
+            node_pools=[
+                cls.create_node_pool_options(p) for p in payload.get("node_pools", ())
+            ],
+            storages=[
+                cls.create_storage_options(type, p) for p in payload.get("storages", ())
+            ],
+            kubernetes_node_pool_id=payload["kubernetes_node_pool_id"],
+            platform_node_pool_id=payload["platform_node_pool_id"],
+            url=URL(url) if url else None,
+            organization=payload.get("organization"),
+            edge_name_template=payload.get("edge_name_template"),
+            edge_external_network_name=payload.get("edge_external_network_name"),
+            catalog_name=payload.get("catalog_name"),
+            storage_profile_names=payload.get("storage_profile_names"),
+        )
+
+    @staticmethod
+    def create_node_pool_options(payload: dict[str, Any]) -> NodePoolOptions:
+        return NodePoolOptions(
+            id=payload["id"],
+            machine_type=payload["machine_type"],
+            cpu=payload["cpu"],
+            available_cpu=payload["available_cpu"],
+            memory=payload["memory"],
+            available_memory=payload["available_memory"],
+            gpu=payload.get("gpu"),
+            gpu_model=payload.get("gpu_model"),
+        )
+
+    @classmethod
+    def create_storage_options(
+        cls, type: CloudProviderType, payload: dict[str, Any]
+    ) -> StorageOptions:
+        if type == CloudProviderType.AWS:
+            return cls.create_aws_storage_options(payload)
+        elif type == CloudProviderType.GCP:
+            return cls.create_google_storage_options(payload)
+        elif type == CloudProviderType.AZURE:
+            return cls.create_azure_storage_options(payload)
+        else:
+            raise ValueError(
+                f"Storage options are not supported for {type.value!r} cloud provider"
+            )
+
+    @staticmethod
+    def create_aws_storage_options(payload: dict[str, Any]) -> AWSStorageOptions:
+        return AWSStorageOptions(
+            id=payload["id"],
+            performance_mode=EFSPerformanceMode(payload["performance_mode"]),
+            throughput_mode=EFSThroughputMode(payload["throughput_mode"]),
+            provisioned_throughput_mibps=payload.get("provisioned_throughput_mibps"),
+        )
+
+    @staticmethod
+    def create_google_storage_options(payload: dict[str, Any]) -> GoogleStorageOptions:
+        return GoogleStorageOptions(
+            id=payload["id"],
+            tier=GoogleFilestoreTier(payload["tier"]),
+            min_capacity=payload["min_capacity"],
+            max_capacity=payload["max_capacity"],
+        )
+
+    @staticmethod
+    def create_azure_storage_options(payload: dict[str, Any]) -> AzureStorageOptions:
+        return AzureStorageOptions(
+            id=payload["id"],
+            tier=AzureStorageTier(payload["tier"]),
+            replication_type=AzureReplicationType(payload["replication_type"]),
+            min_file_share_size=payload["min_file_share_size"],
+            max_file_share_size=payload["max_file_share_size"],
+        )
+
     def create_cluster(self, payload: dict[str, Any]) -> Cluster:
         orchestrator = payload.get("orchestrator")
         storage = payload.get("storage")
@@ -272,7 +373,7 @@ class EntityFactory:
             return self._create_azure_cloud_provider(payload)
         elif cp_type == CloudProviderType.ON_PREM:
             return self._create_on_prem_cloud_provider(payload)
-        elif cp_type.startswith("vcd_"):
+        elif cp_type.is_vcd:
             return self._create_vcd_cloud_provider(payload)
         raise ValueError(f"Cloud provider '{cp_type}' is not supported")
 
@@ -314,19 +415,6 @@ class EntityFactory:
             zones=payload.get("zones", NodePool.zones),
         )
 
-    @staticmethod
-    def create_node_pool_template(payload: dict[str, Any]) -> NodePoolTemplate:
-        return NodePoolTemplate(
-            id=payload["id"],
-            machine_type=payload["machine_type"],
-            cpu=payload["cpu"],
-            available_cpu=payload["available_cpu"],
-            memory=payload["memory"],
-            available_memory=payload["available_memory"],
-            gpu=payload.get("gpu"),
-            gpu_model=payload.get("gpu_model"),
-        )
-
     def _create_aws_storage(self, payload: dict[str, Any]) -> AWSStorage:
         result = AWSStorage(
             id=payload["id"],
