@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from dataclasses import replace
 from datetime import datetime
 from decimal import Decimal
@@ -13,12 +15,15 @@ from neuro_config_client.entities import (
     AWSCloudProvider,
     AWSCredentials,
     AWSStorage,
+    AWSStorageOptions,
     AzureCloudProvider,
     AzureCredentials,
     AzureReplicationType,
     AzureStorage,
+    AzureStorageOptions,
     AzureStorageTier,
     BucketsConfig,
+    CloudProviderOptions,
     CloudProviderType,
     Cluster,
     ClusterLocationType,
@@ -33,6 +38,7 @@ from neuro_config_client.entities import (
     GoogleCloudProvider,
     GoogleFilestoreTier,
     GoogleStorage,
+    GoogleStorageOptions,
     GrafanaCredentials,
     HelmRegistryConfig,
     IdleJobConfig,
@@ -43,6 +49,7 @@ from neuro_config_client.entities import (
     MonitoringConfig,
     NeuroAuthConfig,
     NodePool,
+    NodePoolOptions,
     OnPremCloudProvider,
     OpenStackCredentials,
     OrchestratorConfig,
@@ -57,6 +64,7 @@ from neuro_config_client.entities import (
     TPUPreset,
     TPUResource,
     VCDCloudProvider,
+    VCDCloudProviderOptions,
     VCDCredentials,
     VCDStorage,
     VolumeConfig,
@@ -106,7 +114,7 @@ class TestEntityFactory:
                 "metrics": {"url": "https://secrets-dev.neu.ro"},
                 "disks": {
                     "url": "https://secrets-dev.neu.ro",
-                    "storage_limit_per_user_gb": 1024,
+                    "storage_limit_per_user": 1024,
                 },
                 "ingress": {"acme_environment": "production"},
                 "dns": {
@@ -151,7 +159,7 @@ class TestEntityFactory:
                         "name": "cpu-micro",
                         "credits_per_hour": "10",
                         "cpu": 0.1,
-                        "memory_mb": 100,
+                        "memory": 100,
                     }
                 ],
                 "allow_privileged_mode": True,
@@ -162,7 +170,7 @@ class TestEntityFactory:
                         "name": "idle",
                         "count": 1,
                         "image": "miner",
-                        "resources": {"cpu_m": 1000, "memory_mb": 1024},
+                        "resources": {"cpu_m": 1000, "memory": 1024},
                     },
                     {
                         "name": "idle",
@@ -170,7 +178,7 @@ class TestEntityFactory:
                         "image": "miner",
                         "command": ["bash"],
                         "args": ["-c", "sleep infinity"],
-                        "resources": {"cpu_m": 1000, "memory_mb": 1024},
+                        "resources": {"cpu_m": 1000, "memory": 1024},
                         "env": {"NAME": "VALUE"},
                         "node_selector": {"label": "value"},
                         "image_pull_secret": "secret",
@@ -196,7 +204,7 @@ class TestEntityFactory:
                     name="idle",
                     count=1,
                     image="miner",
-                    resources=Resources(cpu_m=1000, memory_mb=1024),
+                    resources=Resources(cpu_m=1000, memory=1024),
                 ),
                 IdleJobConfig(
                     name="idle",
@@ -204,7 +212,7 @@ class TestEntityFactory:
                     image="miner",
                     command=["bash"],
                     args=["-c", "sleep infinity"],
-                    resources=Resources(cpu_m=1000, memory_mb=1024),
+                    resources=Resources(cpu_m=1000, memory=1024),
                     env={"NAME": "VALUE"},
                     node_selector={"label": "value"},
                     image_pull_secret="secret",
@@ -244,9 +252,9 @@ class TestEntityFactory:
                 "idle_size": 1,
                 "cpu": 4.0,
                 "available_cpu": 3.0,
-                "memory_mb": 12 * 1024,
-                "available_memory_mb": 10 * 1024,
-                "disk_size_gb": 700,
+                "memory": 12 * 1024,
+                "available_memory": 10 * 1024,
+                "disk_size": 700,
                 "gpu": 1,
                 "gpu_model": "nvidia-tesla-k80",
                 "tpu": {
@@ -267,8 +275,8 @@ class TestEntityFactory:
             idle_size=1,
             cpu=4.0,
             available_cpu=3.0,
-            memory_mb=12 * 1024,
-            available_memory_mb=10 * 1024,
+            memory=12 * 1024,
+            available_memory=10 * 1024,
             gpu=1,
             gpu_model="nvidia-tesla-k80",
             tpu=mock.ANY,
@@ -301,12 +309,12 @@ class TestEntityFactory:
                 "name": "cpu-small",
                 "credits_per_hour": "10",
                 "cpu": 4.0,
-                "memory_mb": 1024,
+                "memory": 1024,
             }
         )
 
         assert result == ResourcePreset(
-            name="cpu-small", credits_per_hour=Decimal("10"), cpu=4.0, memory_mb=1024
+            name="cpu-small", credits_per_hour=Decimal("10"), cpu=4.0, memory=1024
         )
 
     def test_create_resource_preset_with_memory_gpu_tpu_preemptible_affinity(
@@ -317,7 +325,7 @@ class TestEntityFactory:
                 "name": "gpu-small",
                 "credits_per_hour": "10",
                 "cpu": 4.0,
-                "memory_mb": 12288,
+                "memory": 12288,
                 "gpu": 1,
                 "gpu_model": "nvidia-tesla-k80",
                 "tpu": {"type": "tpu", "software_version": "v1"},
@@ -331,7 +339,7 @@ class TestEntityFactory:
             name="gpu-small",
             credits_per_hour=Decimal("10"),
             cpu=4.0,
-            memory_mb=12288,
+            memory=12288,
             gpu=1,
             gpu_model="nvidia-tesla-k80",
             tpu=TPUPreset(type="tpu", software_version="v1"),
@@ -353,7 +361,7 @@ class TestEntityFactory:
                 "url": "https://storage-dev.neu.ro",
                 "volumes": [
                     {},
-                    {"path": "/volume", "size_mb": 1024},
+                    {"path": "/volume", "size": 1024},
                 ],
             }
         )
@@ -362,7 +370,7 @@ class TestEntityFactory:
             url=URL("https://storage-dev.neu.ro"),
             volumes=[
                 VolumeConfig(),
-                VolumeConfig(path="/volume", size_mb=1024),
+                VolumeConfig(path="/volume", size=1024),
             ],
         )
 
@@ -422,11 +430,11 @@ class TestEntityFactory:
 
     def test_create_disks(self, factory: EntityFactory) -> None:
         result = factory.create_disks(
-            {"url": "https://metrics-dev.neu.ro", "storage_limit_per_user_gb": 1024}
+            {"url": "https://metrics-dev.neu.ro", "storage_limit_per_user": 1024}
         )
 
         assert result == DisksConfig(
-            url=URL("https://metrics-dev.neu.ro"), storage_limit_per_user_gb=1024
+            url=URL("https://metrics-dev.neu.ro"), storage_limit_per_user=1024
         )
 
     def test_create_buckets(self, factory: EntityFactory) -> None:
@@ -483,9 +491,9 @@ class TestEntityFactory:
                     "max_size": 1,
                     "cpu": 8.0,
                     "available_cpu": 7.0,
-                    "memory_mb": 52 * 1024,
-                    "available_memory_mb": 45 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 52 * 1024,
+                    "available_memory": 45 * 1024,
+                    "disk_size": 700,
                 },
                 {
                     "id": "n1_highmem_32",
@@ -497,9 +505,9 @@ class TestEntityFactory:
                     "idle_size": 1,
                     "cpu": 32.0,
                     "available_cpu": 31.0,
-                    "memory_mb": 208 * 1024,
-                    "available_memory_mb": 201 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 208 * 1024,
+                    "available_memory": 201 * 1024,
+                    "disk_size": 700,
                     "gpu": 1,
                     "gpu_model": "nvidia-tesla-k80",
                     "is_preemptible": True,
@@ -511,8 +519,8 @@ class TestEntityFactory:
                 "backend": "filestore",
                 "tier": "PREMIUM",
                 "instances": [
-                    {"size_mb": 5 * 1024 * 1024, "ready": False},
-                    {"name": "org", "size_mb": 3 * 1024 * 1024, "ready": True},
+                    {"size": 5 * 1024 * 1024, "ready": False},
+                    {"name": "org", "size": 3 * 1024 * 1024, "ready": True},
                 ],
             },
         }
@@ -539,26 +547,28 @@ class TestEntityFactory:
             node_pools=[
                 NodePool(
                     name="n1-highmem-8",
+                    id="n1_highmem_8",
                     machine_type="n1-highmem-8",
                     min_size=0,
                     max_size=1,
                     cpu=8.0,
                     available_cpu=7.0,
-                    memory_mb=52 * 1024,
-                    available_memory_mb=45 * 1024,
-                    disk_size_gb=700,
+                    memory=52 * 1024,
+                    available_memory=45 * 1024,
+                    disk_size=700,
                 ),
                 NodePool(
                     name="n1-highmem-32-1xk80-preemptible",
+                    id="n1_highmem_32",
                     machine_type="n1-highmem-32",
                     min_size=0,
                     max_size=1,
                     idle_size=1,
                     cpu=32.0,
                     available_cpu=31.0,
-                    memory_mb=208 * 1024,
-                    available_memory_mb=201 * 1024,
-                    disk_size_gb=700,
+                    memory=208 * 1024,
+                    available_memory=201 * 1024,
+                    disk_size=700,
                     gpu=1,
                     gpu_model="nvidia-tesla-k80",
                     is_preemptible=True,
@@ -569,8 +579,8 @@ class TestEntityFactory:
                 description="GCP Filestore (Premium)",
                 tier=GoogleFilestoreTier.PREMIUM,
                 instances=[
-                    StorageInstance(size_mb=5 * 1024 * 1024),
-                    StorageInstance(name="org", size_mb=3 * 1024 * 1024, ready=True),
+                    StorageInstance(size=5 * 1024 * 1024),
+                    StorageInstance(name="org", size=3 * 1024 * 1024, ready=True),
                 ],
             ),
         )
@@ -605,9 +615,9 @@ class TestEntityFactory:
                     "max_size": 1,
                     "cpu": 8.0,
                     "available_cpu": 7.0,
-                    "memory_mb": 32 * 1024,
-                    "available_memory_mb": 28 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 32 * 1024,
+                    "available_memory": 28 * 1024,
+                    "disk_size": 700,
                 },
                 {
                     "id": "p2_xlarge_4",
@@ -619,9 +629,9 @@ class TestEntityFactory:
                     "idle_size": 1,
                     "cpu": 4.0,
                     "available_cpu": 3.0,
-                    "memory_mb": 61 * 1024,
-                    "available_memory_mb": 57 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 61 * 1024,
+                    "available_memory": 57 * 1024,
+                    "disk_size": 700,
                     "gpu": 1,
                     "gpu_model": "nvidia-tesla-k80",
                     "is_preemptible": True,
@@ -648,26 +658,28 @@ class TestEntityFactory:
             node_pools=[
                 NodePool(
                     name="m5-2xlarge",
+                    id="m5_2xlarge_8",
                     machine_type="m5.2xlarge",
                     min_size=0,
                     max_size=1,
                     cpu=8.0,
                     available_cpu=7.0,
-                    memory_mb=32 * 1024,
-                    available_memory_mb=28 * 1024,
-                    disk_size_gb=700,
+                    memory=32 * 1024,
+                    available_memory=28 * 1024,
+                    disk_size=700,
                 ),
                 NodePool(
                     name="p2-xlarge-1xk80-preemptible",
+                    id="p2_xlarge_4",
                     machine_type="p2.xlarge",
                     min_size=0,
                     max_size=1,
                     idle_size=1,
                     cpu=4.0,
                     available_cpu=3.0,
-                    memory_mb=61 * 1024,
-                    available_memory_mb=57 * 1024,
-                    disk_size_gb=700,
+                    memory=61 * 1024,
+                    available_memory=57 * 1024,
+                    disk_size=700,
                     gpu=1,
                     gpu_model="nvidia-tesla-k80",
                     is_preemptible=True,
@@ -713,9 +725,9 @@ class TestEntityFactory:
                     "max_size": 1,
                     "cpu": 8.0,
                     "available_cpu": 7.0,
-                    "memory_mb": 32 * 1024,
-                    "available_memory_mb": 28 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 32 * 1024,
+                    "available_memory": 28 * 1024,
+                    "disk_size": 700,
                 },
                 {
                     "id": "standard_nc6_6",
@@ -727,9 +739,9 @@ class TestEntityFactory:
                     "idle_size": 1,
                     "cpu": 6.0,
                     "available_cpu": 5.0,
-                    "memory_mb": 56 * 1024,
-                    "available_memory_mb": 50 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 56 * 1024,
+                    "available_memory": 50 * 1024,
+                    "disk_size": 700,
                     "gpu": 1,
                     "gpu_model": "nvidia-tesla-k80",
                     "is_preemptible": True,
@@ -741,8 +753,8 @@ class TestEntityFactory:
                 "tier": "Premium",
                 "replication_type": "LRS",
                 "instances": [
-                    {"size_mb": 100 * 1024, "ready": False},
-                    {"name": "org", "size_mb": 200 * 1024, "ready": True},
+                    {"size": 100 * 1024, "ready": False},
+                    {"name": "org", "size": 200 * 1024, "ready": True},
                 ],
             },
         }
@@ -761,26 +773,28 @@ class TestEntityFactory:
             node_pools=[
                 NodePool(
                     name="Standard_D8s_v3",
+                    id="standard_d8s_v3_8",
                     machine_type="Standard_D8s_v3",
                     min_size=0,
                     max_size=1,
                     cpu=8.0,
                     available_cpu=7.0,
-                    memory_mb=32 * 1024,
-                    available_memory_mb=28 * 1024,
-                    disk_size_gb=700,
+                    memory=32 * 1024,
+                    available_memory=28 * 1024,
+                    disk_size=700,
                 ),
                 NodePool(
                     name="Standard_NC6-1xk80-preemptible",
+                    id="standard_nc6_6",
                     machine_type="Standard_NC6",
                     min_size=0,
                     max_size=1,
                     idle_size=1,
                     cpu=6.0,
                     available_cpu=5.0,
-                    memory_mb=56 * 1024,
-                    available_memory_mb=50 * 1024,
-                    disk_size_gb=700,
+                    memory=56 * 1024,
+                    available_memory=50 * 1024,
+                    disk_size=700,
                     gpu=1,
                     gpu_model="nvidia-tesla-k80",
                     is_preemptible=True,
@@ -792,8 +806,8 @@ class TestEntityFactory:
                 tier=AzureStorageTier.PREMIUM,
                 replication_type=AzureReplicationType.LRS,
                 instances=[
-                    StorageInstance(size_mb=100 * 1024),
-                    StorageInstance(name="org", size_mb=200 * 1024, ready=True),
+                    StorageInstance(size=100 * 1024),
+                    StorageInstance(name="org", size=200 * 1024, ready=True),
                 ],
             ),
         )
@@ -825,9 +839,9 @@ class TestEntityFactory:
                     "machine_type": "cpu-machine",
                     "cpu": 1.0,
                     "available_cpu": 1.0,
-                    "memory_mb": 1024,
-                    "available_memory_mb": 1024,
-                    "disk_size_gb": 700,
+                    "memory": 1024,
+                    "available_memory": 1024,
+                    "disk_size": 700,
                 },
                 {
                     "role": "platform_job",
@@ -837,9 +851,9 @@ class TestEntityFactory:
                     "machine_type": "gpu-machine-1xk80",
                     "cpu": 1.0,
                     "available_cpu": 1.0,
-                    "memory_mb": 1024,
-                    "available_memory_mb": 1024,
-                    "disk_size_gb": 700,
+                    "memory": 1024,
+                    "available_memory": 1024,
+                    "disk_size": 700,
                     "gpu": 1,
                     "gpu_model": "nvidia-tesla-k80",
                     "price": "0.9",
@@ -862,9 +876,9 @@ class TestEntityFactory:
                     name="cpu-machine",
                     cpu=1.0,
                     available_cpu=1.0,
-                    memory_mb=1024,
-                    available_memory_mb=1024,
-                    disk_size_gb=700,
+                    memory=1024,
+                    available_memory=1024,
+                    disk_size=700,
                     machine_type="cpu-machine",
                 ),
                 NodePool(
@@ -873,9 +887,9 @@ class TestEntityFactory:
                     name="gpu-machine-1xk80",
                     cpu=1.0,
                     available_cpu=1.0,
-                    memory_mb=1024,
-                    available_memory_mb=1024,
-                    disk_size_gb=700,
+                    memory=1024,
+                    available_memory=1024,
+                    disk_size=700,
                     gpu=1,
                     gpu_model="nvidia-tesla-k80",
                     price=Decimal("0.9"),
@@ -921,9 +935,9 @@ class TestEntityFactory:
                     "machine_type": "Master-neuro",
                     "cpu": 8.0,
                     "available_cpu": 7.0,
-                    "memory_mb": 32 * 1024,
-                    "available_memory_mb": 29 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 32 * 1024,
+                    "available_memory": 29 * 1024,
+                    "disk_size": 700,
                 },
                 {
                     "id": "x16_neuro_16",
@@ -934,9 +948,9 @@ class TestEntityFactory:
                     "machine_type": "X16-neuro",
                     "cpu": 16.0,
                     "available_cpu": 15.0,
-                    "memory_mb": 40 * 1024,
-                    "available_memory_mb": 37 * 1024,
-                    "disk_size_gb": 700,
+                    "memory": 40 * 1024,
+                    "available_memory": 37 * 1024,
+                    "disk_size": 700,
                     "gpu": 1,
                     "gpu_model": "nvidia-tesla-k80",
                     "price": "0.9",
@@ -945,10 +959,10 @@ class TestEntityFactory:
             ],
             "storage": {
                 "profile_name": "profile",
-                "size_gib": 10,
+                "size": 10,
                 "instances": [
-                    {"size_mb": 7 * 1024, "ready": False},
-                    {"name": "org", "size_mb": 3 * 1024, "ready": True},
+                    {"size": 7 * 1024, "ready": False},
+                    {"name": "org", "size": 3 * 1024, "ready": True},
                 ],
                 "description": "profile",
             },
@@ -973,23 +987,25 @@ class TestEntityFactory:
                     min_size=1,
                     max_size=1,
                     name="Master-neuro",
+                    id="master_neuro_8",
                     machine_type="Master-neuro",
                     cpu=8.0,
                     available_cpu=7.0,
-                    memory_mb=32 * 1024,
-                    available_memory_mb=29 * 1024,
-                    disk_size_gb=700,
+                    memory=32 * 1024,
+                    available_memory=29 * 1024,
+                    disk_size=700,
                 ),
                 NodePool(
                     min_size=1,
                     max_size=1,
                     name="X16-neuro-1xk80",
+                    id="x16_neuro_16",
                     machine_type="X16-neuro",
                     cpu=16.0,
                     available_cpu=15.0,
-                    memory_mb=40 * 1024,
-                    available_memory_mb=37 * 1024,
-                    disk_size_gb=700,
+                    memory=40 * 1024,
+                    available_memory=37 * 1024,
+                    disk_size=700,
                     gpu=1,
                     gpu_model="nvidia-tesla-k80",
                     price=Decimal("0.9"),
@@ -999,10 +1015,10 @@ class TestEntityFactory:
             storage=VCDStorage(
                 description="profile",
                 profile_name="profile",
-                size_gib=10,
+                size=10,
                 instances=[
-                    StorageInstance(size_mb=7 * 1024),
-                    StorageInstance(name="org", size_mb=3 * 1024, ready=True),
+                    StorageInstance(size=7 * 1024),
+                    StorageInstance(name="org", size=3 * 1024, ready=True),
                 ],
             ),
         )
@@ -1152,6 +1168,196 @@ class TestEntityFactory:
             ),
         )
 
+    @pytest.fixture
+    def node_pool_options_response(self) -> dict[str, Any]:
+        return {
+            "id": "standard_nd24s_24",
+            "machine_type": "Standard_ND24s",
+            "cpu": 24,
+            "available_cpu": 23,
+            "memory": 458752,
+            "available_memory": 452608,
+            "gpu": 4,
+            "gpu_model": "nvidia-tesla-p40",
+            "extra_info": "will be ignored",
+        }
+
+    @pytest.fixture
+    def node_pool_options(self) -> NodePoolOptions:
+        return NodePoolOptions(
+            id="standard_nd24s_24",
+            machine_type="Standard_ND24s",
+            cpu=24,
+            available_cpu=23,
+            memory=458752,
+            available_memory=452608,
+            gpu=4,
+            gpu_model="nvidia-tesla-p40",
+        )
+
+    def test_aws_cloud_provider_options(
+        self,
+        factory: EntityFactory,
+        node_pool_options_response: dict[str, Any],
+        node_pool_options: NodePoolOptions,
+    ) -> None:
+        response = {
+            "node_pools": [node_pool_options_response],
+            "storages": [
+                {
+                    "id": "generalpurpose_bursting",
+                    "performance_mode": "generalPurpose",
+                    "throughput_mode": "bursting",
+                }
+            ],
+        }
+        result = factory.create_cloud_provider_options(CloudProviderType.AWS, response)
+
+        assert result == CloudProviderOptions(
+            type=CloudProviderType.AWS,
+            node_pools=[node_pool_options],
+            storages=[
+                AWSStorageOptions(
+                    id="generalpurpose_bursting",
+                    performance_mode=EFSPerformanceMode.GENERAL_PURPOSE,
+                    throughput_mode=EFSThroughputMode.BURSTING,
+                )
+            ],
+        )
+
+    def test_aws_cloud_provider_options_defaults(self, factory: EntityFactory) -> None:
+        result = factory.create_cloud_provider_options(CloudProviderType.AWS, {})
+
+        assert result == CloudProviderOptions(
+            type=CloudProviderType.AWS, node_pools=[], storages=[]
+        )
+
+    def test_google_cloud_provider_options(
+        self,
+        factory: EntityFactory,
+        node_pool_options_response: dict[str, Any],
+        node_pool_options: NodePoolOptions,
+    ) -> None:
+        response = {
+            "node_pools": [node_pool_options_response],
+            "storages": [
+                {
+                    "id": "standard",
+                    "tier": "STANDARD",
+                    "min_capacity": 1099511627776,
+                    "max_capacity": 70258793014886,
+                }
+            ],
+        }
+        result = factory.create_cloud_provider_options(CloudProviderType.GCP, response)
+
+        assert result == CloudProviderOptions(
+            type=CloudProviderType.GCP,
+            node_pools=[node_pool_options],
+            storages=[
+                GoogleStorageOptions(
+                    id="standard",
+                    tier=GoogleFilestoreTier.STANDARD,
+                    min_capacity=1099511627776,
+                    max_capacity=70258793014886,
+                )
+            ],
+        )
+
+    def test_azure_cloud_provider_options(
+        self,
+        factory: EntityFactory,
+        node_pool_options_response: dict[str, Any],
+        node_pool_options: NodePoolOptions,
+    ) -> None:
+        response = {
+            "node_pools": [node_pool_options_response],
+            "storages": [
+                {
+                    "id": "standard_lrs",
+                    "tier": "Standard",
+                    "replication_type": "LRS",
+                    "min_file_share_size": 1073741824,
+                    "max_file_share_size": 5497558138880,
+                }
+            ],
+        }
+        result = factory.create_cloud_provider_options(
+            CloudProviderType.AZURE, response
+        )
+
+        assert result == CloudProviderOptions(
+            type=CloudProviderType.AZURE,
+            node_pools=[node_pool_options],
+            storages=[
+                AzureStorageOptions(
+                    id="standard_lrs",
+                    tier=AzureStorageTier.STANDARD,
+                    replication_type=AzureReplicationType.LRS,
+                    min_file_share_size=1073741824,
+                    max_file_share_size=5497558138880,
+                )
+            ],
+        )
+
+    def test_vcd_cloud_provider_options_defaults(
+        self,
+        factory: EntityFactory,
+        node_pool_options_response: dict[str, Any],
+        node_pool_options: NodePoolOptions,
+    ) -> None:
+        response = {
+            "node_pools": [node_pool_options_response],
+            "kubernetes_node_pool_id": "master_neuro_2",
+            "platform_node_pool_id": "master_neuro_2",
+        }
+        result = factory.create_cloud_provider_options(
+            CloudProviderType.VCD_MTS, response
+        )
+
+        assert result == VCDCloudProviderOptions(
+            type=CloudProviderType.VCD_MTS,
+            node_pools=[node_pool_options],
+            storages=[],
+            kubernetes_node_pool_id="master_neuro_2",
+            platform_node_pool_id="master_neuro_2",
+        )
+
+    def test_vcd_cloud_provider_options(
+        self,
+        factory: EntityFactory,
+        node_pool_options_response: dict[str, Any],
+        node_pool_options: NodePoolOptions,
+    ) -> None:
+        response = {
+            "node_pools": [node_pool_options_response],
+            "kubernetes_node_pool_id": "master_neuro_2",
+            "platform_node_pool_id": "master_neuro_2",
+            "url": "https://vcd",
+            "organization": "neuro-org",
+            "edge_name_template": "neuro-edge",
+            "edge_external_network_name": "neuro-edge-external",
+            "catalog_name": "neuro",
+            "storage_profile_names": ["neuro-storage"],
+        }
+        result = factory.create_cloud_provider_options(
+            CloudProviderType.VCD_MTS, response
+        )
+
+        assert result == VCDCloudProviderOptions(
+            type=CloudProviderType.VCD_MTS,
+            node_pools=[node_pool_options],
+            storages=[],
+            kubernetes_node_pool_id="master_neuro_2",
+            platform_node_pool_id="master_neuro_2",
+            url=URL("https://vcd"),
+            organization="neuro-org",
+            edge_name_template="neuro-edge",
+            edge_external_network_name="neuro-edge-external",
+            catalog_name="neuro",
+            storage_profile_names=["neuro-storage"],
+        )
+
 
 class TestPayloadFactory:
     @pytest.fixture
@@ -1175,7 +1381,7 @@ class TestPayloadFactory:
                         name="cpu-micro",
                         credits_per_hour=Decimal(10),
                         cpu=0.1,
-                        memory_mb=100,
+                        memory=100,
                     )
                 ],
                 pre_pull_images=["neuromation/base"],
@@ -1184,7 +1390,7 @@ class TestPayloadFactory:
                         name="idle",
                         count=1,
                         image="miner",
-                        resources=Resources(cpu_m=1000, memory_mb=1024),
+                        resources=Resources(cpu_m=1000, memory=1024),
                     ),
                     IdleJobConfig(
                         name="idle",
@@ -1192,7 +1398,7 @@ class TestPayloadFactory:
                         image="miner",
                         command=["bash"],
                         args=["-c", "sleep infinity"],
-                        resources=Resources(cpu_m=1000, memory_mb=1024),
+                        resources=Resources(cpu_m=1000, memory=1024),
                         env={"NAME": "VALUE"},
                         node_selector={"label": "value"},
                         image_pull_secret="secret",
@@ -1218,7 +1424,7 @@ class TestPayloadFactory:
                     "name": "idle",
                     "count": 1,
                     "image": "miner",
-                    "resources": {"cpu_m": 1000, "memory_mb": 1024},
+                    "resources": {"cpu_m": 1000, "memory": 1024},
                 },
                 {
                     "name": "idle",
@@ -1226,7 +1432,7 @@ class TestPayloadFactory:
                     "image": "miner",
                     "command": ["bash"],
                     "args": ["-c", "sleep infinity"],
-                    "resources": {"cpu_m": 1000, "memory_mb": 1024},
+                    "resources": {"cpu_m": 1000, "memory": 1024},
                     "env": {"NAME": "VALUE"},
                     "node_selector": {"label": "value"},
                     "image_pull_secret": "secret",
@@ -1263,9 +1469,9 @@ class TestPayloadFactory:
                 idle_size=1,
                 cpu=4.0,
                 available_cpu=3.0,
-                memory_mb=12 * 1024,
-                available_memory_mb=10 * 1024,
-                disk_size_gb=700,
+                memory=12 * 1024,
+                available_memory=10 * 1024,
+                disk_size=700,
                 gpu=1,
                 gpu_model="nvidia-tesla-k80",
                 tpu=TPUResource(
@@ -1286,9 +1492,9 @@ class TestPayloadFactory:
             "idle_size": 1,
             "cpu": 4.0,
             "available_cpu": 3.0,
-            "memory_mb": 12 * 1024,
-            "available_memory_mb": 10 * 1024,
-            "disk_size_gb": 700,
+            "memory": 12 * 1024,
+            "available_memory": 10 * 1024,
+            "disk_size": 700,
             "gpu": 1,
             "gpu_model": "nvidia-tesla-k80",
             "tpu": {
@@ -1307,13 +1513,13 @@ class TestPayloadFactory:
         assert result == {
             "name": "node-pool",
             "available_cpu": 1.0,
-            "available_memory_mb": 1024,
+            "available_memory": 2**30,
             "cpu": 1.0,
-            "disk_size_gb": 150,
+            "disk_size": 150 * 2**30,
             "idle_size": 0,
             "is_preemptible": False,
             "max_size": 1,
-            "memory_mb": 1024,
+            "memory": 2**30,
             "min_size": 0,
         }
 
@@ -1336,7 +1542,7 @@ class TestPayloadFactory:
                 name="cpu-small",
                 credits_per_hour=Decimal("10"),
                 cpu=4.0,
-                memory_mb=1024,
+                memory=1024,
             )
         )
 
@@ -1344,7 +1550,7 @@ class TestPayloadFactory:
             "name": "cpu-small",
             "credits_per_hour": "10",
             "cpu": 4.0,
-            "memory_mb": 1024,
+            "memory": 1024,
         }
 
     def test_create_resource_preset_with_memory_gpu_tpu_preemptible_affinity(
@@ -1355,7 +1561,7 @@ class TestPayloadFactory:
                 name="gpu-small",
                 credits_per_hour=Decimal("10"),
                 cpu=4.0,
-                memory_mb=12288,
+                memory=12288,
                 gpu=1,
                 gpu_model="nvidia-tesla-k80",
                 tpu=TPUPreset(type="tpu", software_version="v1"),
@@ -1369,7 +1575,7 @@ class TestPayloadFactory:
             "name": "gpu-small",
             "credits_per_hour": "10",
             "cpu": 4.0,
-            "memory_mb": 12288,
+            "memory": 12288,
             "gpu": 1,
             "gpu_model": "nvidia-tesla-k80",
             "tpu": {"type": "tpu", "software_version": "v1"},
@@ -1390,7 +1596,7 @@ class TestPayloadFactory:
                 url=URL("https://storage-dev.neu.ro"),
                 volumes=[
                     VolumeConfig(),
-                    VolumeConfig(path="/volume", size_mb=1024),
+                    VolumeConfig(path="/volume", size=1024),
                 ],
             )
         )
@@ -1399,7 +1605,7 @@ class TestPayloadFactory:
             "url": "https://storage-dev.neu.ro",
             "volumes": [
                 {},
-                {"path": "/volume", "size_mb": 1024},
+                {"path": "/volume", "size": 1024},
             ],
         }
 
@@ -1471,13 +1677,13 @@ class TestPayloadFactory:
     def test_create_disks(self, factory: PayloadFactory) -> None:
         result = factory.create_disks(
             DisksConfig(
-                url=URL("https://metrics-dev.neu.ro"), storage_limit_per_user_gb=1024
+                url=URL("https://metrics-dev.neu.ro"), storage_limit_per_user=1024
             )
         )
 
         assert result == {
             "url": "https://metrics-dev.neu.ro",
-            "storage_limit_per_user_gb": 1024,
+            "storage_limit_per_user": 1024,
         }
 
     def test_create_buckets(self, factory: PayloadFactory) -> None:
@@ -1616,3 +1822,77 @@ class TestPayloadFactory:
             "neuro_registry": {"username": "username", "password": "password"},
             "neuro_helm": {"username": "username", "password": "password"},
         }
+
+    @pytest.fixture
+    def node_pool(self) -> NodePool:
+        return NodePool(
+            name="my-node-pool",
+            id="someid",
+            min_size=0,
+            max_size=10,
+            idle_size=1,
+            machine_type="some-machine-type",
+            cpu=10,
+            available_cpu=9,
+            memory=2048,
+            available_memory=1024,
+            disk_size=100500,
+            disk_type="some-disk-type",
+            gpu=1,
+            gpu_model="some-gpu-model",
+            price=Decimal(180),
+            currency="rabbits",
+            is_preemptible=True,
+            zones=("here", "there"),
+        )
+
+    def test_node_pool(self, factory: PayloadFactory, node_pool: NodePool) -> None:
+        payload = factory.create_node_pool(node_pool)
+        assert payload == {
+            "id": "someid",
+            "name": "my-node-pool",
+            "role": "platform_job",
+            "min_size": 0,
+            "max_size": 10,
+            "idle_size": 1,
+            "is_preemptible": True,
+            "machine_type": "some-machine-type",
+            "cpu": 10,
+            "available_cpu": 9,
+            "memory": 2048,
+            "available_memory": 1024,
+            "disk_size": 100500,
+            "disk_type": "some-disk-type",
+            "gpu": 1,
+            "gpu_model": "some-gpu-model",
+            "zones": ("here", "there"),
+            "price": "180",
+            "currency": "rabbits",
+        }
+
+        np = replace(
+            node_pool,
+            cpu=None,
+            available_cpu=None,
+            memory=None,
+            available_memory=None,
+            zones=None,
+        )
+
+        payload = factory.create_node_pool(np)
+        assert payload == {
+            "id": "someid",
+            "name": "my-node-pool",
+            "role": "platform_job",
+            "min_size": 0,
+            "max_size": 10,
+            "idle_size": 1,
+            "is_preemptible": True,
+            "machine_type": "some-machine-type",
+            "disk_size": 100500,
+            "disk_type": "some-disk-type",
+            "gpu": 1,
+            "gpu_model": "some-gpu-model",
+            "price": "180",
+            "currency": "rabbits",
+        }
